/*  Copyright (c) 2002, 2003, 2006 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Andy Adamson <andros@citi.umich.edu>
 *  David M. Richter <richterd@citi.umich.edu>
 *  Alexis Mackenzie <allamack@citi.umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <sys/types.h>
#include <acl/libacl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <getopt.h>
#include "libacl_nfs4.h"

void usage(const char *);

int main(int argc, char** argv)
{
	char *path = argv[argc - 1];
	int result;
	struct nfs4_acl * nacl;
	struct stat st;
	char * savefile_s = NULL;
	FILE *fd;
	int option_index = 1;
	int opt;
	struct option opt_a[] = { {"help", 0, NULL, 'h'}};
	char *execname = argv[0];

	while (1) {
		opt = getopt_long(argc, argv, "f:h", opt_a, NULL);
		if (opt == -1) {
			/* Allow one undefined option - it's the filename. */
			if (option_index < (argc - 1)) {
				while (option_index < argc)
					printf("Unrecognized option: %s\n", argv[option_index++]);
				goto usage_failed;
			}
			break;
		}
		switch (opt) {
			case 'h':
				goto usage_failed;

			case 'f':
				if (savefile_s != NULL) {
					printf("Output file specified twice. Aborting.\n");
					goto usage_failed;
				}
				savefile_s = optarg;
				break;

			default:
				goto usage_failed;
		}
		option_index = optind;
	}

	if (argc < 2) {
		printf("You must specify a path\n");
		goto usage_failed;
	}

	if (savefile_s != NULL) {
		result = stat(savefile_s, &st);
		if (result == 0)
			printf("Warning: output file already exists\n");
		fd = fopen(savefile_s, "w+");
		if (fd == NULL) {
			printf("Failed opening output file %s\n", savefile_s);
			result = 1;
			goto out;
		}
	} else
		fd = stdout;

	nacl = nfs4_acl_for_path(path);
	if (nacl == NULL) {
		result = 1;
		goto out;
	}
	nfs4_print_acl(stdout, nacl);
	result = 0;

out:
	return result;

usage_failed:
	usage(execname);
	return 1;
}

void usage(const char *name)
{
	printf("usage: %s [-f <outputfile>] <path>\n", name);
	printf("Prints the NFSv4 acl entry for the given path.\n");
	printf("	Permission letter mapping:\n");
	printf("	r - NFS4_ACE_READ_DATA\n");
	printf("	w - NFS4_ACE_WRITE_DATA\n");
	printf("	a - NFS4_ACE_APPEND_DATA\n");
	printf("	x - NFS4_ACE_EXECUTE\n");
	printf("	d - NFS4_ACE_DELETE\n");
	printf("	l - NFS4_ACE_LIST_DIRECTORY\n");
	printf("	f - NFS4_ACE_ADD_FILE\n");
	printf("	s - NFS4_ACE_ADD_SUBDIRECTORY\n");
	printf("	n - NFS4_ACE_READ_NAMED_ATTRS\n");
	printf("	N - NFS4_ACE_WRITE_NAMED_ATTRS\n");
	printf("	D - NFS4_ACE_DELETE_CHILD\n");
	printf("	t - NFS4_ACE_READ_ATTRIBUTES\n");
	printf("	T - NFS4_ACE_WRITE_ATTRIBUTES\n");
	printf("	c - NFS4_ACE_READ_ACL\n");
	printf("	C - NFS4_ACE_WRITE_ACL\n");
	printf("	o - NFS4_ACE_WRITE_OWNER\n");
	printf("	y - NFS4_ACE_SYNCHRONIZE\n");
}
